@@ -78,7 +78,7 @@ pathlib = _LazyModule.load('pathlib', 'pathlib')
 platform = _LazyModule.load('platform', 'platform')
 functools = _LazyModule.load('functools', 'functools')
 subprocess = _LazyModule.load('subprocess', 'subprocess')
-__version__ = "0.2.5"
+__version__ = "0.2.6"
 
 
 @functools.lru_cache(1)
@@ -292,6 +292,18 @@ function _get_complete_statement {{
     complete -p {cmd} 2> /dev/null || echo "-F _minimal"
 }}
 
+function getarg {{
+    find=$1
+    shift 1
+    prev=""
+    for i in $* ; do
+        if [ "$prev" = "$find" ] ; then
+            echo $i
+        fi
+        prev=$i
+    done
+}}
+
 _complete_stmt=$(_get_complete_statement)
 if echo "$_complete_stmt" | grep --quiet -e "_minimal"
 then
@@ -299,15 +311,20 @@ then
     _complete_stmt=$(_get_complete_statement)
 fi
 
-_func=$(echo "$_complete_stmt" | grep -o -e '-F \w\+' | cut -d ' ' -f 2)
-declare -f "$_func" > /dev/null || exit 1
+# Is -C (subshell) or -F (function) completion used?
+if [[ $_complete_stmt =~ "-C" ]] ; then
+    _func=$(eval getarg "-C" $_complete_stmt)
+else
+    _func=$(eval getarg "-F" $_complete_stmt)
+    declare -f "$_func" > /dev/null || exit 1
+fi
 
 echo "$_complete_stmt"
-COMP_WORDS=({line})
-COMP_LINE={comp_line}
-COMP_POINT=${{#COMP_LINE}}
-COMP_COUNT={end}
-COMP_CWORD={n}
+export COMP_WORDS=({line})
+export COMP_LINE={comp_line}
+export COMP_POINT=${{#COMP_LINE}}
+export COMP_COUNT={end}
+export COMP_CWORD={n}
 $_func {cmd} {prefix} {prev}
 
 # print out completions, right-stripped if they contain no internal spaces
@@ -1343,6 +1360,7 @@ def complete_dir(prefix, line, start, end, ctx, cdpath=False):
 inspect = _LazyModule.load('inspect', 'inspect')
 # amalgamated builtins
 importlib = _LazyModule.load('importlib', 'importlib')
+warnings = _LazyModule.load('warnings', 'warnings')
 cabc = _LazyModule.load('collections', 'collections.abc', 'cabc')
 # amalgamated xonsh.tools
 # amalgamated xonsh.lazyasd
@@ -1519,6 +1537,18 @@ def complete_python_mode(prefix, line, start, end, ctx):
     return set(prefix_start + i for i in python_matches)
 
 
+def _turn_off_warning(func):
+    """Decorator to turn off warning temporarily."""
+
+    def wrapper(*args, **kwargs):
+        warnings.filterwarnings("ignore")
+        r = func(*args, **kwargs)
+        warnings.filterwarnings("once", category=DeprecationWarning)
+        return r
+
+    return wrapper
+
+
 def _safe_eval(expr, ctx):
     """Safely tries to evaluate an expression. If this fails, it will return
     a (None, None) tuple.
@@ -1537,6 +1567,7 @@ def _safe_eval(expr, ctx):
     return val, _ctx
 
 
+@_turn_off_warning
 def attr_complete(prefix, ctx, filter_func):
     """Complete attributes of an object."""
     attrs = set()
@@ -1578,6 +1609,7 @@ def attr_complete(prefix, ctx, filter_func):
     return attrs
 
 
+@_turn_off_warning
 def python_signature_complete(prefix, line, end, ctx, filter_func):
     """Completes a python function (or other callable) call by completing
     argument and keyword argument names.
